#!/usr/bin/env python3
from taseq.params import Params

pm = Params('default')
args = pm.set_options()

class Default(object):
    def __init__(self, args):
        self.args = args

def main():
    Default(args)

if __name__ == '__main__':
    main()



import argparse
import os
import sys
from taseq.__init__ import __version__

class Params(object):

    def __init__(self, program_name):
        self.program_name = program_name

    def set_options(self):
        if self.program_name == 'hapcall':
            parser = self.hapcall_options()
        elif self.program_name == 'genotype':
            parser = self.genotype_options()
        elif self.program_name == 'filter':
            parser = self.filter_options()
        elif self.program_name == 'draw':
            parser = self.draw_options()
        elif self.program_name == 'default':
            parser = self.default_options()

        if len(sys.argv) == 1:
            args = parser.parse_args(['-h'])
        else:
            args = parser.parse_args()
        return args
    
    def hapcall_options(self):
        parser = argparse.ArgumentParser(description='taseq version {}'.format(__version__),formatter_class=argparse.RawTextHelpFormatter)
        parser.usage = ('taseq_hapcall -I <Directory containing input FASTQ>\n'
                        '              -R <File of reference FASTA>\n'
                        '              -V <File of target VCF>\n'
                        '              -n <Name of output directory>\n'
                        '              ... \n')

        # set options
        parser.add_argument('-I', '--input',
                            action='store',
                            required=True,
                            type=str,
                            help=('Directory containing input FASTQ.\n'
                                  'This directory must contain only fastq file used in genotyping.\n'
                                  'The names of fastq files must be unique for each [Name]_*R1* and [Name]_*R2*.\n'
                                  'The string to the left of the first underscore is considered the sample name.\n'
                                  'gzip (fastq.gz) supported.\n'),
                            metavar='')
        
        parser.add_argument('-R', '--ref',
                            action='store',
                            required=True,
                            type=str,
                            help='File of reference genome (fasta).',
                            metavar='')
        
        parser.add_argument('-V', '--vcf',
                            action='store',
                            required=True,
                            type=str,
                            help=('VCF File containing only target SNPs.\n'
                                  '(VCF made by mkselect is recommended.)\n'),
                            metavar='')
        
        parser.add_argument('-n', '--name',
                            action='store',
                            default='taseq_hapcall',
                            type=str,
                            help=('Name of output directory.\n'
                                  'Already existing directory name is not allowed.\n'
                                  'Default: taseq_hapcall.\n'),
                            metavar='')

        parser.add_argument('--cpu',
                            action='store',
                            default=2,
                            type=int,
                            help=('Number of CPUs to use.\n'
                                  'Default: 2.\n'),
                            metavar='')
        
        parser.add_argument('--adapter',
                            action='store',
                            default='NONE',
                            choices=['NONE', 'NEXTERA', 'TRUSEQ', 'CUSTOM'],
                            help=('Adapter sequences used for trimming fastq.\n'
                                  'NONE means the input fastq has already trimmed.\n'
                                  'When CUSTOM designated, --adapterfile must be specified.\n'
                                  'Choose from [NONE, NEXTERA, TRUSEQ, CUSTOM].\n'
                                  'Default: NONE.\n'),
                            metavar='')
        
        parser.add_argument('--adapterfile',
                            action='store',
                            type=str,
                            help=('This is valid when --adapter = CUSTOM.\n'),
                            metavar='')
        
        parser.add_argument('--seqlen',
                            action='store',
                            default=150,
                            type=int,
                            help=('Sequence length of fastq.\n'
                                  '3 dash bases over this length will be cut.\n'
                                  'Default: 150.\n'),
                            metavar='')
        
        parser.add_argument('--minlen',
                            action='store',
                            default=60,
                            type=int,
                            help=('Ignore reads which are shorter than this value after trimming.\n'
                                  'Default: 60.\n'),
                            metavar='')
        
        parser.add_argument('--quality_threshold',
                            action='store',
                            default=30,
                            type=int,
                            help=('If the quality of the bases at both ends of a read\n'
                                  'is below this threshold, it is deleted.\n'
                                  'Default: 30.\n'),
                            metavar='')
        
        # set version
        parser.add_argument('-v', '--version',
                            action='version',
                            version='%(prog)s {}'.format(__version__))
        return parser

    def genotype_options(self):
        parser = argparse.ArgumentParser(description='taseq version {}'.format(__version__),formatter_class=argparse.RawTextHelpFormatter)
        parser.usage = ('taseq_genotype -I <VCF file which is the output of taseq_hapcall>\n'
                        '               -p1 <Parent name genotyped as A>\n'
                        '               -p2 <Parent name genotyped as B>\n'
                        '               -n <Name of output directory>\n'
                        '               ... \n')

        # set options
        parser.add_argument('-I', '--input',
                            action='store',
                            required=True,
                            type=str,
                            help='VCF file which is the output of taseq_hapcall.',
                            metavar='')
        
        parser.add_argument('-p1', '--parent1',
                            action='store',
                            required=True,
                            type=str,
                            help=('Parent name genotyped as A.\n'
                                  'Use the name of vcf column in the input file of taseq_hapcall.\n'),
                            metavar='')
        
        parser.add_argument('-p2', '--parent2',
                            action='store',
                            required=True,
                            type=str,
                            help=('Parent name genotyped as B.\n'
                                  'Use the name of vcf column in the input file of taseq_hapcall.\n'),
                            metavar='')
        
        parser.add_argument('-n', '--name',
                            action='store',
                            default='taseq_genotype',
                            type=str,
                            help=('Name of output directory.\n'
                                  'Already existing directory name is not allowed.\n'
                                  'Default: taseq_genotype.\n'),
                            metavar='')
        
        parser.add_argument('--mindep',
                            action='store',
                            default=6,
                            type=int,
                            help=('Minimum depth to genotype.\n'
                                  'Variants with depth lower than this\n'
                                  'will be genotyped as missing.\n'
                                  'Default: 6.\n'),
                            metavar='')

        parser.add_argument('--hetero_chi',
                            action='store',
                            default=3.84,
                            type=float,
                            help=('Threshold value of chi-square when genotyping as hetero.\n'
                                  'Default value is 3.84 (the threshold for p=0.05).\n'),
                            metavar='')
        
        parser.add_argument('--noise_level',
                            action='store',
                            default=0.1,
                            type=float,
                            help=('When genotyping as homozygous, minor reads below this ratio will be ignored.\n'
                                  'Default: 0.1.\n'),
                            metavar='')

        # set version
        parser.add_argument('-v', '--version',
                            action='version',
                            version='%(prog)s {}'.format(__version__))
        return parser

    def filter_options(self):
        parser = argparse.ArgumentParser(description='taseq version {}'.format(__version__),formatter_class=argparse.RawTextHelpFormatter)
        parser.usage = ('taseq_filter -I <TSV file which is the output of taseq_genotype>\n'
                        '             --parent_sample1 <Parent sample expected to be A>\n'
                        '             --parent_sample2 <Parent sample expected to be B>\n'
                        '             -n <Name of output directory>\n'
                        '             ... \n')

        # set options
        parser.add_argument('-I', '--input',
                            action='store',
                            required=True,
                            type=str,
                            help='TSV file which is the output of taseq_genotype.',
                            metavar='')
        
        parser.add_argument('--parent_sample1',
                            action='store',
                            default=None,
                            type=str,
                            help=('Parent sample expected to be genotype A.\n'
                                  'This must be specified if parental lines are included in your samples.\n'),
                            metavar='')
        
        parser.add_argument('--parent_sample2',
                            action='store',
                            default=None,
                            type=str,
                            help=('Parent sample expected to be genotype B.\n'
                                  'This must be specified if parental lines are included in your samples.\n'),
                            metavar='')
        
        parser.add_argument('-n', '--name',
                            action='store',
                            default='taseq_filter',
                            type=str,
                            help=('Name of output directory.\n'
                                  'Already existing directory name is not allowed.\n'
                                  'Default: taseq_filter.\n'),
                            metavar='')
        
        parser.add_argument('--missing_rate',
                            action='store',
                            default=0.2,
                            type=float,
                            help=('Markers with more missing than this\n'
                                  'value will be removed\n'
                                  'Default: 0.2.\n'),
                            metavar='')

        parser.add_argument('--check_parents',
                            action='store_true',
                            help=('Test the genotype of the parent line.\n'
                                  'If they are inconsistent with the predicted genotype, the marker will be removed.\n'
                                  'This is invalid if -p1 and -p2 are not specified.\n'))

        parser.add_argument('--minor_freq',
                            action='store',
                            default=0,
                            type=float,
                            help=('Threshold of minor allele frequency (MAF).\n'
                                  'Markers whose MAF are lower than this,\n'
                                  'they are removed.\n'
                                  'Default: 0.\n'),
                            metavar='')

        # set version
        parser.add_argument('-v', '--version',
                            action='version',
                            version='%(prog)s {}'.format(__version__))
        return parser
    
    def draw_options(self):
        parser = argparse.ArgumentParser(description='taseq version {}'.format(__version__),formatter_class=argparse.RawTextHelpFormatter)
        parser.usage = ('taseq_draw -I <TSV file which is the output of taseq_filter>\n'
                        '           -F <FASTA Index file to draw chromosome>\n'
                        '           -n <Name of output directory>\n'
                        '           ... \n')

        # set options
        parser.add_argument('-I', '--input',
                            action='store',
                            required=True,
                            type=str,
                            help='TSV file which is the output of taseq_filter.',
                            metavar='')

        parser.add_argument('-F', '--fai',
                            action='store',
                            required=True,
                            type=str,
                            help='FASTA Index file to draw chromosome.',
                            metavar='')
        
        parser.add_argument('-n', '--name',
                            action='store',
                            default='taseq_draw',
                            type=str,
                            help=('Name of output directory.\n'
                                  'Already existing directory name is not allowed.\n'
                                  'Default: taseq_draw.\n'),
                            metavar='') 
        
        parser.add_argument('--color_A',
                            action='store',
                            default='orange',
                            type=str,
                            help='Color of genotype A (Default: orange).\n'
                                 'Limited to color names that can be specified in matplotlib.\n'
                                 '(The same applies below.)',
                            metavar='')
        
        parser.add_argument('--color_B',
                            action='store',
                            default='blue',
                            type=str,
                            help='Color of genotype B (Default: blue).',
                            metavar='')

        parser.add_argument('--color_het',
                            action='store',
                            default='cyan',
                            type=str,
                            help='Color of heterozygous (Default: cyan).',
                            metavar='')

        parser.add_argument('--color_miss',
                            action='store',
                            default='gray',
                            type=str,
                            help='Color of missing genotype (Default: gray).\n',
                            metavar='')

        parser.add_argument('--name_A',
                            action='store',
                            default='A',
                            type=str,
                            help='Name of line A (Default: A).',
                            metavar='')
        
        parser.add_argument('--name_B',
                            action='store',
                            default='B',
                            type=str,
                            help='Name of line B (Default: B).',
                            metavar='')

        # set version
        parser.add_argument('-v', '--version',
                            action='version',
                            version='%(prog)s {}'.format(__version__))
        return parser

    def default_options(self):
        parser = argparse.ArgumentParser(description='taseq version {}'.format(__version__),formatter_class=argparse.RawTextHelpFormatter)
        parser.usage = ('Commands that can be executed with taseq are as follows:\n'
                        '  taseq_hapcall\n'
                        '  taseq_genotype\n'
                        '  taseq_filter\n'
                        '  taseq_draw\n')
        # set version
        parser.add_argument('-v', '--version',
                            action='version',
                            version='%(prog)s {}'.format(__version__))
        return parser

    def hapcall_check_args(self, args):
        #Does the same name directory exist?
        if os.path.isdir(args.name):
            sys.stderr.write(('  Output directory already exist.\n'
                              '  Please rename the --name.\n'))
            sys.exit(1)
        
        #input fastq
        if not os.path.isdir(args.input):
            sys.stderr.write(('  Input fastq directory does not exist.\n'))
            sys.exit(1)
        #input reference genome
        if not os.path.isfile(args.ref):
            sys.stderr.write(('  Input reference genome does not exist.\n'))
            sys.exit(1)
        #input vcf
        if not os.path.isfile(args.vcf):
            sys.stderr.write(('  Input VCF does not exist.\n'))
            sys.exit(1)




#!/usr/bin/env python3
import os
import sys
import pandas as pd
from taseq.params import Params
from taseq.utils import read_vcf, time_for_filename, time_stamp

pm = Params('genotype')
args = pm.set_options()

class Genotype(object):
    def __init__(self, args):
        self.args = args
        self.p1 = args.parent1
        self.p2 = args.parent2
        self.mindep = args.mindep
        self.hchi = args.hetero_chi
        self.noise = args.noise_level
        #This is the name of output directory
        self.dir = args.name
        os.mkdir(self.dir)

        #make input VCF to dataframe
        tmp = read_vcf(args.input)
        self.header = tmp[0] #List
        self.col = tmp[1] #List
        self.vcf = pd.DataFrame(tmp[2], columns=self.col) #dataframe

        self.s_cols = [] #Columun number of samples
        self.p1_col = -1 #Columun number of templates
        self.p2_col = -1

        self.out = [] #output frame
        self.out_dep = [] #output frame

    def command(self):
        #Output command info
        command = ' '.join(sys.argv)
        fn = '{}/command.txt'.format(self.dir)
        with open(fn, 'w') as f:
            f.write('{}\n'.format(command))

    def checkargs(self):
        flag1 = False #Check parents name
        flag2 = False #Check parents name
        for i in range(9, len(self.col)):
            n = self.col[i]
            if n[0:9] == 'template_':
                if n[9:] == self.p1:
                    flag1 = True
                    self.p1_col = i
                if n[9:] == self.p2:
                    flag2 = True
                    self.p2_col = i
            else:
                self.s_cols.append(i)
        if flag1 and flag2:
            pass
        else:
            print(time_stamp(), 
                  '!!ERROR!! Names of parents are wrong.\n', flush=True)
            sys.exit(1)

    def makeframe(self):
        all_list = list(self.vcf['#CHROM'])
        chr_list = []
        pos_list = []
        for l in all_list:
            tmp1 = l.split(':')
            chr = tmp1[0]
            tmp2 = tmp1[1].split('-')
            pos = int((int(tmp2[0]) + int(tmp2[1])) / 2)
            chr_list.append(chr)
            pos_list.append(pos)
        chr_list = pd.Series(data = chr_list, name = 'CHR')
        pos_list = pd.Series(data = pos_list, name = 'POS')
        self.out = pd.concat([chr_list, pos_list], axis=1)
        self.out_dep = self.out.copy()

        #Make the names of markers
        names = []
        for i in range(len(self.out)):
            chr = self.out.iloc[i, 0]
            pos = int(self.out.iloc[i, 1])
            pos_n = f"{round(pos / 10000):04}" #Fill by 0 to be 4 digits
            name = '{}_{}'.format(chr, pos_n)
            names.append(name)
        names = pd.Series(data = names, name = 'NAME')
        self.out = pd.concat([self.out, names], axis=1)

    def convert(self):
        #get Genotypes for each sample
        for i in self.s_cols:
            sn = self.vcf.columns[i]
            res = []
            dep = []
            for j in range(len(self.vcf)):
                #Check haplotype calling format.
                format = self.vcf.iloc[j, 8].split(':')
                gt_index = format.index('GT')
                ad_index = format.index('AD')

                #which genotype A or B
                gt_p1 = self.vcf.iloc[j, self.p1_col].split(':')[gt_index]
                gt_p2 = self.vcf.iloc[j, self.p2_col].split(':')[gt_index]
                if gt_p1 == '0/0' and gt_p2 == '1/1':
                    key = 1 #'0/0' = 'A', '1/1' = 'B'
                elif gt_p1 == '1/1' and gt_p2 == '0/0':
                    key = 2 #'0/0' = 'B', '1/1' = 'A'
                else:
                    res.append('-')
                    dep.append(0)
                    continue

                #analyze for sample genotyping
                data_s = self.vcf.iloc[j, i].split(':') #ex. ['0/0', '12,0', ....]
                if len(data_s) != len(format):
                    res.append('-')
                    dep.append(0)
                    continue
                ad_s = data_s[ad_index].split(',') #ex. [12, 0]
                ref = int(ad_s[0])
                alt = int(ad_s[1])
                dep.append(ref + alt)

                if ref + alt < self.mindep:
                    res.append('-') #Thin depth marker is missing
                    continue

                #Is this marker HOMO?
                minor_freq = min([ref, alt]) / (ref + alt) #ex. [2, 18] --> 0.1
                if minor_freq <= self.noise: #default: noise=0.1
                    #HOMO
                    if key == 1:#'0/0' = 'A', '1/1' = 'B'
                        if ref > alt: #'0/0' = 'A'
                            res.append('A')
                        else: #'1/1' = 'B'
                            res.append('B')
                    if key == 2:#'0/0' = 'B', '1/1' = 'A'
                        if ref > alt: #'0/0' = 'B'
                            res.append('B')
                        else: #'1/1' = 'A'
                            res.append('A')
                else:
                    #Hetero or Missing
                    #chi-squared test
                    tv = (ref + alt) / 2; #theorical varue when segregeted 1:1
                    chi2 = ((ref - tv)**2 + (alt - tv)**2) / tv #chi-squared

                    if ref < 5 or alt < 5:
                        #chi2 test is not suitable for either value is under 5
                        res.append('-')
                    elif chi2 < self.hchi:
                        #Hetero
                        res.append('H')
                    else:
                        res.append('-')
            
            res = pd.Series(data = res, name = sn)
            dep = pd.Series(data = dep, name = sn)
            self.out = pd.concat([self.out, res], axis=1)
            self.out_dep = pd.concat([self.out_dep, dep], axis=1)
        
        #write tsv
        fn = '{}/result_taseq_genotype.tsv'.format(self.dir)
        self.out.to_csv(fn, sep='\t', mode='w', index=False)
        fn = '{}/result_taseq_genotype_depth.tsv'.format(self.dir)
        self.out_dep.to_csv(fn, sep='\t', mode='w', index=False)

def main():
    print(time_stamp(), 'taseq_genotype started.', flush=True)

    prog = Genotype(args)
    prog.command()
    prog.checkargs()
    prog.makeframe()
    prog.convert()

    print(time_stamp(), 'taseq_genotype successfully finished.\n', flush=True)

if __name__ == '__main__':
    main()



import os
import sys
import subprocess as sbp
from taseq.utils import time_stamp, prepare_cmd, call_log


class RefIndex(object):

    def __init__(self, ref, dir):
        self.ref = ref
        self.dir = dir

    def run(self):
        print(time_stamp(),
              'Indexing reference fasta.',
              flush=True)

        cmd1 = 'samtools faidx {} \
                >> {}/log/samtools.log \
                2>&1'.format(self.ref, self.dir)
        cmd1 = prepare_cmd(cmd1)
        try:
            sbp.run(cmd1,
                    stdout=sbp.DEVNULL,
                    stderr=sbp.DEVNULL,
                    shell=True,
                    check=True)
        except sbp.CalledProcessError:
            call_log(self.dir, 'samtools', cmd1)
            sys.exit(1)


        if os.path.isfile('{}.dict'.format(os.path.splitext(self.ref)[0])):
            pass
        else:
            cmd2 = 'picard CreateSequenceDictionary R={} \
                    >> {}/log/picard.log \
                    2>&1'.format(self.ref, self.dir)
            cmd2 = prepare_cmd(cmd2)
            try:
                sbp.run(cmd2,
                        stdout=sbp.DEVNULL,
                        stderr=sbp.DEVNULL,
                        shell=True,
                        check=True)
            except sbp.CalledProcessError:
                call_log(self.dir, 'picard', cmd2)
                sys.exit(1)

        if os.path.isfile('{}.bwt'.format(self.ref)):
            pass
        else:
            cmd3 = 'bwa index {} \
                    >> {}/log/bwa.log \
                    2>&1'.format(self.ref, self.dir)
            cmd3 = prepare_cmd(cmd3)
            try:
                sbp.run(cmd3,
                        stdout=sbp.DEVNULL,
                        stderr=sbp.DEVNULL,
                        shell=True,
                        check=True)
            except sbp.CalledProcessError:
                call_log(self.dir, 'bwa', cmd3)
                sys.exit(1)

        print(time_stamp(),
              'Done.',
              flush=True)


import os
import sys
from taseq.utils import time_stamp

class Adapter(object):

    def __init__(self, outdir, adapter, adapterfile):
        self.dir = outdir
        self.ad = adapter
        self.adfile = adapterfile
        
    def nextera(self):
        a = ('>adapter/1\n'
             'CTGTCTCTTATACACATCT\n'
             '>adapter/2\n'
             'CTGTCTCTTATACACATCT\n')
        return a
    
    def truseq(self):
        a = ('>adapter/1\n'
             'AGATCGGAAGAGCACACGTCTGAACTCCAGTCA\n'
             '>adapter/2\n'
             'AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGT\n')
        return a
    
    def custom(self):
        if not os.path.isfile(self.adfile):
            print(time_stamp(), 
                  '!!ERROR!! Custom adapter file does not exist.\n', flush=True)
            sys.exit(1)
        with open(self.adfile, 'r') as f:
            a = f.read()
        return a
    
    def run(self):
        filename = '{}/adapter/adapter.fa'.format(self.dir)
        with open(filename, 'w') as f:
            if self.ad == 'NEXTERA':
                f.write(self.nextera())
            elif self.ad == 'TRUSEQ':
                f.write(self.truseq())
            elif self.ad == 'CUSTOM':
                f.write(self.custom())




import sys
import subprocess as sbp

import pandas as pd
from taseq.utils import read_vcf, time_stamp, prepare_cmd, call_log

class Mapping(object):

    def __init__(self, dir, samplename):
        self.dir = dir
        self.sn = samplename

    def run(self):
        #Mapping reads using bwa mem
        cmd1 = 'bwa mem -t 1 \
                -R "@RG\\tID:{0}\\tLB:{0}\\tPL:ILLUMINA\\tSM:{0}" \
                {1}/ref/extracted_ref.fasta \
                {1}/fastq/{0}_R1_trimmed.fastq.gz \
                {1}/fastq/{0}_R2_trimmed.fastq.gz \
                > {1}/bam/{0}_aligned_reads.sam'.format(self.sn, self.dir)
        cmd1 = prepare_cmd(cmd1)
        try:
            sbp.run(cmd1, stdout=sbp.DEVNULL, stderr=sbp.DEVNULL,
                    shell=True, check=True)
        except sbp.CalledProcessError:
            print(time_stamp(), 
                  '!!ERROR!! Mapping command 1 was failed.\n', flush=True)
            sys.exit(1)

        #Sort sam and convert to bam
        cmd2 = 'samtools sort -@ 1 -O bam \
                -o {1}/bam/{0}_aligned_reads.bam \
                {1}/bam/{0}_aligned_reads.sam'.format(self.sn, self.dir)
        cmd2 = prepare_cmd(cmd2)
        try:
            sbp.run(cmd2, stdout=sbp.DEVNULL, stderr=sbp.DEVNULL,
                    shell=True, check=True)
        except sbp.CalledProcessError:
            print(time_stamp(), 
                  '!!ERROR!! Mapping command 2 was failed.\n', flush=True)
            sys.exit(1)

        #Remove SAM
        cmd3 = 'rm {1}/bam/{0}_aligned_reads.sam'.format(self.sn, self.dir)
        cmd3 = prepare_cmd(cmd3)
        try:
            sbp.run(cmd3, stdout=sbp.DEVNULL, stderr=sbp.DEVNULL,
                    shell=True, check=True)
        except sbp.CalledProcessError:
            print(time_stamp(), 
                  '!!ERROR!! Mapping command 3 was failed.\n', flush=True)
            sys.exit(1)

        #Add Index to BAM
        cmd4 = 'samtools index {1}/bam/{0}_aligned_reads.bam'.format(self.sn, self.dir)
        cmd4 = prepare_cmd(cmd4)
        try:
            sbp.run(cmd4, stdout=sbp.DEVNULL, stderr=sbp.DEVNULL,
                    shell=True, check=True)
        except sbp.CalledProcessError:
            print(time_stamp(), 
                  '!!ERROR!! Mapping command 4 was failed.\n', flush=True)
            sys.exit(1)

        #Extract only the primary mapped reads
        cmd5 = 'samtools view -@ 1 -bh -F 256 {1}/bam/{0}_aligned_reads.bam \
                > {1}/bam/{0}_aligned_reads_primary.bam'.format(self.sn, self.dir)
        cmd5 = prepare_cmd(cmd5)
        try:
            sbp.run(cmd5, stdout=sbp.DEVNULL, stderr=sbp.DEVNULL,
                    shell=True, check=True)
        except sbp.CalledProcessError:
            print(time_stamp(), 
                  '!!ERROR!! Mapping command 5 was failed.\n', flush=True)
            sys.exit(1)

        #Add Index to BAM
        cmd6 = 'samtools index {1}/bam/{0}_aligned_reads_primary.bam'.format(self.sn, self.dir)
        cmd6 = prepare_cmd(cmd6)
        try:
            sbp.run(cmd6, stdout=sbp.DEVNULL, stderr=sbp.DEVNULL,
                    shell=True, check=True)
        except sbp.CalledProcessError:
            print(time_stamp(), 
                  '!!ERROR!! Mapping command 6 was failed.\n', flush=True)
            sys.exit(1)

        #Make VCF
        cmd7 = 'gatk HaplotypeCaller \
                -R {1}/ref/extracted_ref.fasta \
                -I {1}/bam/{0}_aligned_reads_primary.bam \
                -O {1}/vcf/{0}_raw_variants.vcf \
                --alleles {1}/ref/target.vcf'.format(self.sn, self.dir)
        cmd7 = prepare_cmd(cmd7)
        try:
            sbp.run(cmd7, stdout=sbp.DEVNULL, stderr=sbp.DEVNULL,
                    shell=True, check=True)
        except sbp.CalledProcessError:
            print(time_stamp(), 
                  '!!ERROR!! Mapping command 7 was failed.\n', flush=True)
            sys.exit(1)
        

        #Extract only target variants from VCF
        tmp = read_vcf('{}/ref/target.vcf'.format(self.dir))
        tmpvcf = pd.DataFrame(tmp[2], columns=tmp[1])

        #Raw VCF has many non-target variants
        raw = read_vcf('{1}/vcf/{0}_raw_variants.vcf'.format(self.sn, self.dir))
        rawvcf = pd.DataFrame(raw[2], columns=raw[1])

        #Make Extracted vcf from template VCF
        newvcf = tmpvcf.iloc[:, 0:10].copy()
        newvcf.iloc[:, 5:10] = '.'
        #Modify name of sample column
        newvcf = newvcf.rename(columns={newvcf.columns[9] : self.sn})

        for i in range(len(newvcf)):
            for j in range(len(rawvcf)):
                #Compare values of #CHROM,POS,REF,ALT
                row_new = ','.join(list(newvcf.iloc[i, 0:4]))
                row_raw = ','.join(list(rawvcf.iloc[j, 0:4]))
                if row_new == row_raw:
                    newvcf.iloc[i, :] = rawvcf.iloc[j, :]
                    break

        #Output new VCF
        fn = '{1}/vcf/{0}_selected_variants.vcf'.format(self.sn, self.dir)
        with open(fn, 'w') as f:
            f.write('##fileformat=VCFv4.2\n')
        newvcf.to_csv(fn, sep='\t', mode='a', index=False)

        #add Index to vcf
        cmd8 = 'gatk IndexFeatureFile \
                -I {} \
                >> {}/log/gatk.log 2>&1'.format(fn, self.dir)
        cmd8 = prepare_cmd(cmd8)
        try:
            sbp.run(cmd8,
                    stdout=sbp.DEVNULL,
                    stderr=sbp.DEVNULL,
                    shell=True,
                    check=True)
        except sbp.CalledProcessError:
            call_log(self.dir, 'gatk', cmd8)
            sys.exit(1)



import glob
import sys
import subprocess as sbp
from taseq.utils import prepare_cmd, call_log, time_stamp


class Trimmomatic(object):

    def __init__(self, fqdir, samplename, outdir, crop, minlen, qual, adapter):
        self.fqdir = fqdir
        self.sn = samplename
        self.dir = outdir
        self.crop = crop
        self.minlen = minlen
        self.qual = qual
        self.ad = adapter

    def run(self):
        #Check the input files are pair end
        tmp = ['{}/{}_*R1*'.format(self.fqdir, self.sn), '{}/{}_*R2*'.format(self.fqdir, self.sn)]
        filelist_1 = glob.glob(tmp[0])
        filelist_2 = glob.glob(tmp[1])
        if len(filelist_1) != 1 or len(filelist_2) != 1:
            print(time_stamp(), 
                  '!!ERROR!! Program could not parse the name of input fastq.\n', flush=True)
            sys.exit(1)
            

        if self.ad == 'NONE':
            cmd1 = 'trimmomatic PE -threads 1 -phred33 \
                    {0}/{1}_*R1* \
                    {0}/{1}_*R2* \
                    {2}/fastq/{1}_R1_trimmed.fastq.gz \
                    {2}/fastq/{1}_R1_unpaired.fastq.gz \
                    {2}/fastq/{1}_R2_trimmed.fastq.gz \
                    {2}/fastq/{1}_R2_unpaired.fastq.gz \
                    LEADING:{5} \
                    TRAILING:{5} \
                    CROP:{3} \
                    MINLEN:{4} \
                    >> {2}/log/trimmmomatic.log \
                    2>&1'.format(self.fqdir, self.sn, self.dir, 
                                self.crop, self.minlen, self.qual)
            cmd1 = prepare_cmd(cmd1)

            try:
                sbp.run(cmd1,
                        stdout=sbp.DEVNULL,
                        stderr=sbp.DEVNULL,
                        shell=True,
                        check=True)
            except sbp.CalledProcessError:
                call_log(self.dir, 'trimmmomatic', cmd1)
                sys.exit(1)
        else:
            cmd1 = 'trimmomatic PE -threads 1 -phred33 \
                    {0}/{1}_*R1* \
                    {0}/{1}_*R2* \
                    {2}/fastq/{1}_R1_trimmed.fastq.gz \
                    {2}/fastq/{1}_R1_unpaired.fastq.gz \
                    {2}/fastq/{1}_R2_trimmed.fastq.gz \
                    {2}/fastq/{1}_R2_unpaired.fastq.gz \
                    ILLUMINACLIP:{2}/adapter/adapter.fa:2:30:10 \
                    LEADING:{5} \
                    TRAILING:{5} \
                    CROP:{3} \
                    MINLEN:{4} \
                    >> {2}/log/trimmmomatic.log \
                    2>&1'.format(self.fqdir, self.sn, self.dir, 
                                self.crop, self.minlen, self.qual)
            cmd1 = prepare_cmd(cmd1)

            try:
                sbp.run(cmd1,
                        stdout=sbp.DEVNULL,
                        stderr=sbp.DEVNULL,
                        shell=True,
                        check=True)
            except sbp.CalledProcessError:
                call_log(self.dir, 'trimmmomatic', cmd1)
                sys.exit(1)




#!/usr/bin/env python3
from multiprocessing import Pool
import sys
import subprocess as sbp
import os

import pandas as pd
from taseq.adapter import Adapter
from taseq.extractref import ExtractRef
from taseq.mapping import Mapping
from taseq.mergevcf import MergeVcf
from taseq.params import Params
from taseq.refindex import RefIndex
from taseq.trimmomatic import Trimmomatic
from taseq.utils import prepare_cmd, read_vcf, time_for_filename, time_stamp

pm = Params('hapcall')
args = pm.set_options()


class Hapcall(object):
    def __init__(self, args):
        pm.hapcall_check_args(args)
        self.args = args
        self.ref = args.ref
        self.input = args.input #fastq directry
        self.seqlen =  args.seqlen
        self.minlen = args.minlen
        self.qual = args.quality_threshold
        self.adapter = args.adapter
        self.cpu = args.cpu

        #This is the name of output directory
        self.dir = args.name

        #get sample list
        tmp_fqlist = os.listdir(self.input)
        self.slist = [] #list of sample names
        for f in tmp_fqlist:
            #The strings left of 1st '_' are considered to be sample name
            self.slist.append(f.split('_')[0])
        self.slist = list(set(self.slist)) #extract unique values
        self.slist.sort()
        print(time_stamp(), 
              '{} samples are processing.'.format(len(self.slist)), flush=True)
        
        #get VCF as dataframe
        tmp_vcf = read_vcf(args.vcf)
        self.vcf_head = tmp_vcf[0] #List
        self.vcf_col = tmp_vcf[1] #List
        self.vcf = pd.DataFrame(tmp_vcf[2], columns=self.vcf_col) #dataframe

    def setdir(self):
        cmd1 = ('mkdir {0} {0}/log {0}/bam {0}/vcf '
                '{0}/adapter {0}/fastq {0}/ref').format(self.dir)
        cmd1 = prepare_cmd(cmd1)
        try:
            sbp.run(cmd1,
                    stdout=sbp.DEVNULL,
                    stderr=sbp.DEVNULL,
                    shell=True,
                    check=True)
        except sbp.CalledProcessError:
            print(time_stamp(), 
                  '!!ERROR!! Makeing output directory was failed.\n', flush=True)
            sys.exit(1)

    def command(self):
        #Output command info
        command = ' '.join(sys.argv)
        fn = '{}/command.txt'.format(self.dir)
        with open(fn, 'w') as f:
            f.write('{}\n'.format(command))

    def refindex(self):
        ri = RefIndex(self.ref, self.dir)
        ri.run()

    def setadapter(self):
        ad = Adapter(self.dir, self.adapter, self.args.adapterfile)
        ad.run()

    def trimmomatic(self, num):
        ri = Trimmomatic(self.input, self.slist[num], self.dir, 
                        self.seqlen, self.minlen, self.qual, self.adapter)
        ri.run()

    def extractref(self):
        er = ExtractRef(self.ref, self.vcf, self.dir)
        er.run()

    def mapping(self, num):
        ma = Mapping(self.dir, self.slist[num])
        ma.run()

    def mergevcf(self):
        ma = MergeVcf(self.dir, self.slist)
        ma.run()

def main():
    print(time_stamp(), 'taseq_hapcall started.', flush=True)

    prog = Hapcall(args)
    prog.setdir()
    prog.command()
    prog.refindex()
    prog.setadapter()

    print(time_stamp(), 'Trimming sequences...', flush=True)
    with Pool(prog.cpu) as p:
        p.map(prog.trimmomatic, range(len(prog.slist)))
    print(time_stamp(), 'Done.', flush=True)
        
    prog.extractref()

    print(time_stamp(), 'Mapping fastq to target sequences...', flush=True)
    with Pool(prog.cpu) as p:
        p.map(prog.mapping, range(len(prog.slist)))
    print(time_stamp(), 'Done.', flush=True)

    prog.mergevcf()

    print(time_stamp(), 'taseq_hapcall successfully finished.\n', flush=True)

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import math
import os
import sys

import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as patches

from taseq.params import Params
from taseq.utils import time_for_filename, time_stamp

pm = Params('draw')
args = pm.set_options()

class Draw(object):
    def __init__(self, args):
        self.args = args
        self.input = args.input
        self.fai = args.fai
        self.col_a = args.color_A
        self.col_b = args.color_B
        self.col_h = args.color_het
        self.col_m = args.color_miss
        self.name_a = args.name_A
        self.name_b = args.name_B

        #This is the name of output directory
        self.dir = args.name
        os.mkdir(self.dir)

        #make input TSV to dataframe
        self.data = pd.read_csv(args.input, sep='\t') #dataframe
        self.data = self.data[self.data['JUDGE']]

        #Prepare chromosome information
        self.fai_data = []
        with open(self.fai, 'r') as f:
            for row in f:
                row = row.strip()
                self.fai_data.append(row.split('\t'))
        self.fai_col = ['chr', 'len', 'A', 'B', 'C']
        self.fai_data = pd.DataFrame(self.fai_data, columns=self.fai_col)
        self.fai_data['len'] = self.fai_data['len'].astype(int)
        
    def command(self):
        #Output command info
        command = ' '.join(sys.argv)
        fn = '{}/command.txt'.format(self.dir)
        with open(fn, 'w') as f:
            f.write('{}\n'.format(command))
            
    def run(self):
        print(time_stamp(),
              'Drawing positions of selected markers.',
              flush=True)

        #number of digits in the length of the longest chromosome
        digits = math.floor(math.log10(max(self.fai_data['len'])))
        standard = 10**(digits)
        #if the longest chr length is 23098790,
        #digits = 7
        #standard = 10000000

        if(max(self.fai_data['len']) / standard < 2):
            standard = standard / 5
        elif(max(self.fai_data['len']) / standard < 5):
            standard = int(standard / 2)
        #if the longest chr length is 23098790,
        #standard = 5000000

        y_axis_at = range(0, standard*11, standard)
        y_axis_lab = []
        if(standard >= 100000):
            st_lab = standard/1000000
            sign = 'M'
        elif(standard >= 100):
            st_lab = standard/1000
            sign = 'K'
        else:
            st_lab = standard
            sign = 'bp'

        for i in range(11):
            y_axis_lab.append('{}{}'.format(round(st_lab * i, 1), sign))

        longest_len = max(self.fai_data['len'])



        sample_col = range(3, len(self.data.columns) - 12)
        for h in sample_col:
            # Create a figure
            fig = plt.figure(figsize=(5, 5), dpi=144)
            ax = fig.add_subplot(111,
                                xlim=[-1, len(self.fai_data['chr'])],
                                xticks=range(len(self.fai_data['chr'])),
                                xticklabels=self.fai_data['chr'],
                                xlabel="Chromosome",
                                ylim=[longest_len*1.05, -longest_len*0.05],
                                yticks=y_axis_at,
                                yticklabels=y_axis_lab,
                                ylabel="Position")
            plt.subplots_adjust(left=0.15, right=0.95, bottom=0.15, top=0.95)
            plt.xticks(rotation=45)
            plt.xlim(-1, len(self.fai_data['chr']))
            plt.ylim(longest_len*1.05, -longest_len*0.05)

            legends = []
            legends.append(patches.Patch(color=self.col_a, label=self.name_a))
            legends.append(patches.Patch(color=self.col_b, label=self.name_b))
            legends.append(patches.Patch(color=self.col_h, label='Hetero'))
            legends.append(patches.Patch(color=self.col_m, label='Missing'))
            ax.legend(handles=legends, loc="lower right")


            for i in range(len(self.fai_data['chr'])):
                #Draw rectangle of chromosome
                r = patches.Rectangle(xy=(i-0.4, 0), width=0.8,
                    height=self.fai_data['len'][i], ec=None, fc='lightgray', fill=True)
                ax.add_patch(r)

                ##make data matrix
                data_select = self.data[self.data['CHR'] == self.fai_data['chr'][i]]
                data_select.reset_index(inplace=True, drop=True)

                #Draw rectangle
                for j in range(len(data_select)):
                    pos = data_select['POS'][j]
                    geno = data_select[data_select.columns[h]][j]

                    if(j == 0):
                        sta = 0
                    else:
                        sta = (data_select['POS'][j-1] + pos) / 2
                    if(j == len(data_select) - 1):
                        end = self.fai_data['len'][i]
                    else:
                        end = (data_select['POS'][j+1] + pos) / 2

                    if(geno == 'A'):
                        r = patches.Rectangle(xy=(i-0.4, sta), width=0.8, height=end - sta,
                                ec=None, fc=self.col_a, fill=True)
                    elif(geno == 'B'):
                        r = patches.Rectangle(xy=(i-0.4, sta), width=0.8, height=end - sta,
                                ec=None, fc=self.col_b, fill=True)
                    elif(geno == 'H'):
                        r = patches.Rectangle(xy=(i-0.4, sta), width=0.8, height=end - sta,
                                ec=None, fc=self.col_h, fill=True)
                    else:
                        r = patches.Rectangle(xy=(i-0.4, sta), width=0.8, height=end - sta,
                                ec=None, fc=self.col_m, fill=True)
                    ax.add_patch(r)
                    plt.hlines(pos, i-0.4, i+0.4, color='black', lw=1)

                #Draw rectangle of chromosome
                r = patches.Rectangle(xy=(i-0.4, 0), width=0.8,
                    height=self.fai_data['len'][i], ec='black', fill=False)
                ax.add_patch(r)

            # Save figure
            file = '{}/{}.png'.format(self.dir, self.data.columns[h])
            fig.savefig(file, dpi=144)

            # Release memory
            plt.clf()
            plt.close()

        print(time_stamp(),
              'Done.',
              flush=True)

def main():
    prog = Draw(args)
    prog.command()
    prog.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from setuptools import setup, find_packages # type: ignore
from taseq.__init__ import __version__

setup(name='taseq',
      version=__version__,
      description='Downstream analysis for targetted amplicon sequencing.',
      author='Koki Chigira',
      author_email='kyoujin2009kutar@gmail.com',
      url='https://github.com/KChigira/taseq/',
      license='MIT',
      packages=find_packages(),
      install_requires=[
        'pandas',
        'matplotlib',
      ],
      entry_points={'console_scripts': [
            'taseq_hapcall = taseq.hapcall:main',
            'taseq_genotype = taseq.genotype:main',
            'taseq_filter = taseq.filter:main',
            'taseq_draw = taseq.draw:main',
            'taseq = taseq.default:main',
            ]
      }
    )



import sys
import subprocess as sbp
import pandas as pd
from taseq.refindex import RefIndex
from taseq.utils import time_stamp, prepare_cmd, call_log


class ExtractRef(object):

    def __init__(self, ref, vcf, dir):
        self.ref = ref
        self.vcf = pd.DataFrame(vcf)
        self.dir = dir

    def run(self):
        print(time_stamp(),
              'Extracting reference sequence data.',
              flush=True)

        #VCF for only target variant.
        #Modify column names of samples
        new_vcf = self.vcf
        for i in range(9, len(new_vcf.columns)):
            cn = new_vcf.columns[i]
            newcn = 'template_{}'.format(cn)
            new_vcf = new_vcf.rename(columns={cn : newcn})

        #Making target refarence
        fn_newref = '{}/ref/extracted_ref.fasta'.format(self.dir)
        for i in range(len(self.vcf)):
            target_chr = self.vcf.at[self.vcf.index[i], '#CHROM']
            target_pos = int(self.vcf.at[self.vcf.index[i], 'POS'])
            #Make strings such as "chr01:100000-100400"
            query = '{}:{}-{}'.format(target_chr,
                                      target_pos - 200,
                                      target_pos + 200)
            cmd1 = 'samtools faidx {} {} \
                    >> {}'.format(self.ref, query, fn_newref)
            cmd1 = prepare_cmd(cmd1)
            try:
                sbp.run(cmd1,
                        stdout=sbp.DEVNULL,
                        stderr=sbp.DEVNULL,
                        shell=True,
                        check=True)
            except sbp.CalledProcessError:
                call_log(self.dir, 'samtools', cmd1)
                sys.exit(1)

            #Modify the #CHROM and POS of target VCF
            new_vcf.at[new_vcf.index[i], '#CHROM'] = query
            new_vcf.at[new_vcf.index[i], 'POS'] = 201

        #Add index to new reference
        ri = RefIndex(fn_newref, self.dir)
        ri.run()

        #Output new VCF
        fn = '{}/ref/target.vcf'.format(self.dir)
        with open(fn, 'w') as f:
            f.write('##fileformat=VCFv4.2\n')
        new_vcf.to_csv(fn, sep='\t', mode='a', index=False)
        #add Index to vcf
        cmd2 = 'gatk IndexFeatureFile \
                -I {} \
                >> {}/log/gatk.log 2>&1'.format(fn, self.dir)
        cmd2 = prepare_cmd(cmd2)
        try:
            sbp.run(cmd2,
                    stdout=sbp.DEVNULL,
                    stderr=sbp.DEVNULL,
                    shell=True,
                    check=True)
        except sbp.CalledProcessError:
            call_log(self.dir, 'gatk', cmd2)
            sys.exit(1)

        print(time_stamp(),
              'Done.',
              flush=True)


#!/usr/bin/env python3
import os
import sys
import pandas as pd
from taseq.params import Params
from taseq.utils import time_for_filename, time_stamp

pm = Params('filter')
args = pm.set_options()

class Filter(object):
    def __init__(self, args):
        self.args = args
        self.p1 = args.parent_sample1
        self.p2 = args.parent_sample2
        self.missrate = args.missing_rate
        self.mfreq = args.minor_freq
       
        #This is the name of output directory
        self.dir = args.name
        os.mkdir(self.dir)

        #make input TSV to dataframe
        self.data = pd.read_csv(args.input, sep='\t') #dataframe

        self.s_cols = [] #Columun number of samples
        self.p1_col = -1 #Columun number of parent sample1
        self.p2_col = -1 #Columun number of parent sample2

        self.out = [] #output frame

        #Below are all for judge remove or remain
        self.A_list = []
        self.B_list = []
        self.H_list = []
        self.M_list = []
        self.A_ratio = []
        self.B_ratio = []
        self.H_ratio = []
        self.M_ratio = []
        self.judge_missrate = [] #boolean
        self.judge_mfreq = [] #boolean
        self.judge_parent = []
        self.judge_all = []

    def command(self):
        #Output command info
        command = ' '.join(sys.argv)
        fn = '{}/command.txt'.format(self.dir)
        with open(fn, 'w') as f:
            f.write('{}\n'.format(command))

    def checkargs(self):
        if self.p1 is None or self.p2 is None:
            self.s_cols = range(3, len(self.data.columns))
            return
        
        flag1 = False #Check parents name
        flag2 = False #Check parents name
        for i in range(3, len(self.data.columns)):
            n = self.data.columns[i]
            if n == self.p1:
                flag1 = True
                self.p1_col = i
            elif n == self.p2:
                flag2 = True
                self.p2_col = i
            else:
                self.s_cols.append(i)
        if flag1 and flag2:
            pass
        else:
            print(time_stamp(), 
                  '!!ERROR!! Names of parents are wrong.\n', flush=True)
            sys.exit(1)

    def countalleles(self):
        for i in range(len(self.data)):
            A_cnt = 0
            B_cnt = 0
            H_cnt = 0
            M_cnt = 0
            for j in self.s_cols:
                allele = self.data.iloc[i, j]
                if allele == 'A':
                    A_cnt = A_cnt + 1
                elif allele == 'B':
                    B_cnt = B_cnt + 1
                elif allele == 'H':
                    H_cnt = H_cnt + 1
                else:
                    M_cnt = M_cnt + 1
            self.A_list.append(A_cnt)
            self.B_list.append(B_cnt)
            self.H_list.append(H_cnt)
            self.M_list.append(M_cnt)
            valid = (A_cnt + B_cnt + H_cnt)
            if valid == 0:
                A_freq = 0
                B_freq = 0
                H_freq = 0
            else:
                A_freq = A_cnt / (A_cnt + B_cnt + H_cnt)
                B_freq = B_cnt / (A_cnt + B_cnt + H_cnt)
                H_freq = H_cnt / (A_cnt + B_cnt + H_cnt)
            Missrate = M_cnt / len(self.s_cols)
            self.A_ratio.append(A_freq)
            self.B_ratio.append(B_freq)
            self.H_ratio.append(H_freq)
            self.M_ratio.append(Missrate)
            if A_freq < self.mfreq or B_freq < self.mfreq:
                self.judge_mfreq.append(False)
            else:
                self.judge_mfreq.append(True)
            if Missrate > self.missrate:
                self.judge_missrate.append(False)
            else:
                self.judge_missrate.append(True)

    def checkparents(self):
        for i in range(len(self.data)):
            if self.p1 is None or self.p2 is None or not(self.args.check_parents):
                self.judge_parent.append(True)
                continue
            
            p1_allele = self.data.iloc[i, self.p1_col]
            p2_allele = self.data.iloc[i, self.p2_col]
            if (p1_allele == 'A' and p2_allele == 'A') or \
               (p1_allele == 'B' and p2_allele == 'B'):
                self.judge_parent.append(False)
            else:
                self.judge_parent.append(True)

    def judge(self):
        cnt_true = 0
        for i in range(len(self.data)):
            if self.judge_mfreq[i] and self.judge_missrate[i] and self.judge_parent[i]:
                self.judge_all.append(True)
                cnt_true = cnt_true + 1
            else:
                self.judge_all.append(False)
        print(time_stamp(), 
              '{} / {} markers are passed.'.format(cnt_true, len(self.data)), flush=True)

    def makeoutput(self):
        add = [self.A_list, self.B_list, self.H_list, self.M_list,
               self.A_ratio, self.B_ratio, self.H_ratio, self.M_ratio,
               self.judge_missrate, self.judge_mfreq,
               self.judge_parent, self.judge_all]
        addcol = ['Count_A', 'Count_B', 'Count_H', 'Count_Missing',
                  'Ratio_A', 'Ratio_B', 'Ratio_H', 'Missing_Rate',
                  'JUDGE_missrate', 'JUDGE_minor_allele',
                  'JUDGE_parents', 'JUDGE']
        addframe = pd.DataFrame(add)
        addframe = addframe.T #inverse
        addframe.columns = addcol
        output1 = pd.concat([self.data, addframe], axis=1)
        #write tsv
        fn = '{}/result_taseq_filter.tsv'.format(self.dir)
        output1.to_csv(fn, sep='\t', mode='w', index=False)

        #format for R/qtl
        #Make dataset of number of chrom.
        chrnum =  []
        chr = 0
        current = ''
        for l in list(output1['CHR']):
            if l != current:
                chr = + chr + 1
                current = l
            chrnum.append(chr)
        chrnum = pd.Series(data = chrnum, name = 'chr')

        #Col[0]= Marker name, Col[1]=number of chrom., Col[2]~=Only sample genotype
        output2 = pd.concat([output1['NAME'], chrnum, output1[output1.columns[self.s_cols]]], axis=1)
        #delete removed markers
        output2 = output2[self.judge_all]

        #inverse
        output2 = output2.T.copy()
        output2 = output2.reset_index() #new column 'index' generated
        output2.iloc[0, 0] = 'id'
        output2.iloc[1, 0] = ''
        fn = '{}/result_taseq_filter_formated_for_Rqtl.csv'.format(self.dir)
        output2.to_csv(fn, sep=',', mode='w', index=False, header=False)

def main():
    print(time_stamp(), 'taseq_filter started.', flush=True)

    prog = Filter(args)
    prog.command()
    prog.checkargs()
    prog.countalleles()
    prog.checkparents()
    prog.judge()
    prog.makeoutput()

    print(time_stamp(), 'taseq_filter successfully finished.\n', flush=True)

if __name__ == '__main__':
    main()



import sys
import pandas as pd
from taseq.utils import time_stamp, read_vcf


class MergeVcf(object):

    def __init__(self, dir, samplelist):
        self.dir = dir
        self.sl = samplelist

    def run(self):
        print(time_stamp(), 'Merging vcf files.', flush=True)

        #Prepare frame of VCF from template
        tmp = read_vcf('{}/ref/target.vcf'.format(self.dir))
        mergedvcf = pd.DataFrame(tmp[2], columns=tmp[1])

        #Add genotypes for each sample
        for i in range(len(self.sl)):
            fn = '{}/vcf/{}_selected_variants.vcf'.format(self.dir, self.sl[i])
            tmp = read_vcf(fn)
            svcf = pd.DataFrame(tmp[2], columns=tmp[1])
            mergedvcf = pd.concat([mergedvcf, svcf[svcf.columns[9]]], axis=1)
            #row 9 is genotypes

        #Add command info
        command = ' '.join(sys.argv)

        #Output new VCF
        fn = '{}/result_taseq_hapcall.vcf'.format(self.dir)
        with open(fn, 'w') as f:
            f.write('##fileformat=VCFv4.2\n')
            f.write('##command={}\n'.format(command))
        mergedvcf.to_csv(fn, sep='\t', mode='a', index=False)

        print(time_stamp(),
              'Done.',
              flush=True)

